using AutoMapper;
using Connect.Controllers;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Connect.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.EntityFrameworkCore;
using Old = Connect.Models.Legacy;
using Microsoft.AspNetCore.Authorization;
using System.Security.Claims;
using Connect.Areas.Invoicing.ViewModels;
using Connect.Services;

namespace Connect.Areas.Invoicing.Controllers
{
    [Area("Invoicing")]
    //   [Route("Areas/Invoicing/InvoiceHeader")]
    [Authorize(Policy = "InvoiceStaffOnly")]

    public class InvoiceHeadersController : SecuredControllerBase<AdminContext>
    {
        IMailService EmailService;
        public Old.LegacyContext LegacyContext { get; }

        public InvoiceHeadersController(AdminContext context, IMapper mapper, Old.LegacyContext legacyContext, IMailService mailService) : base(context, mapper)
        {
            LegacyContext = legacyContext;
            EmailService = mailService;
        }

        // GET: Invoices
        [Route("Invoicing/Invoices")]
        public async Task<IActionResult> Index()
        {
            ViewBag.Employees = await LegacyContext.Personnels
                .OrderBy(r => r.Name)
                .Select(r => new LookupEntry<int>()
                {
                    Text = $"{r.Name} ({r.LinkBlue})",
                    Id = r.PriKey
                })
                .AsNoTracking()
                .ToListAsync();

            ViewBag.Clients = await LegacyContext.ClientContacts
                .Include(r => r.Client)
                .Select(r => new LookupEntry<int>()
                {
                    Text = $"{r.ContactName}: {r.Client.Organization}",
                    Id = r.ClientContactId
                })
                .AsNoTracking()
                .ToListAsync();

            ViewBag.Invoices = await LegacyContext.InvoiceHeaders
                .Select(r => new LookupEntry<int>()
                {
                    Text = $"{r.InvoiceNumber.Trim()}",
                    Id = r.InvoiceHeaderId
                })
                .AsNoTracking()
                .ToListAsync();

            //var records = Mapper.Map<List<InvoiceHeaderViewModel>>(headers);

            return View();
        }

        // GET: Invoices/Details/5
        [HttpGet("Invoicing/Invoices/{id}")]
        public async Task<IActionResult> Details(string id)
        {
            if (id == null)
            {
                return NotFound();
            }

            id = id.Trim();
            
            Old.InvoiceHeader invoiceHeader = await LegacyContext.InvoiceHeaders.SingleAsync(m => m.InvoiceNumber.Trim() == id);
            if (invoiceHeader == null)
            {
                return NotFound();
            }

            ViewBag.Contact = await LegacyContext.ClientContacts
                .Include(r=> r.Client)
                .Where(p => p.ClientContactId == invoiceHeader.ClientContactId)
                .AsNoTracking()
                .FirstOrDefaultAsync();

            ViewBag.Submitter = await LegacyContext.Personnels
                .Where(p => p.PriKey == invoiceHeader.PersonnelKey)
                .AsNoTracking()
                .FirstOrDefaultAsync();

            ViewBag.Header = invoiceHeader;

            ViewBag.InvoiceDetailUnits = await Context.InvoiceDetailUnits
                .OrderBy(r => r.Unit)
                .Select(r => r.Unit)
                .AsNoTracking()
                .ToListAsync();

            ViewBag.InvoiceDetailServices = await Context.InvoiceDetailServices
                .OrderBy(r => r.Service)
                .Select(r => r.Service)
                .AsNoTracking()
                .ToListAsync();

            var fy = DateTime.Now.ToAdminFiscalYear();

            ViewBag.ValidAccounts = await LegacyContext.Projects
                .Where(r => r.FiscalYear == fy && r.ProjectNumber.StartsWith("E186"))
                .Select(r => r.ProjectNumber)
                .AsNoTracking()
                .ToListAsync();

                return View(invoiceHeader);
        }

    }
}
